##############################################################################
#
# Simplified example of cross object validation.
#
# All the NFS clients check that the NFS servers that they use indeed export
# the directories that they mount. This is done transparently by adding some
# validation code to the mount record type. Further checks such as wildcards
# in export list or export/mount options mismatch are left as an exercise for
# the reader ;-)
#
# Here is how to compile the server and two clients (result on stdout):
#   % pan --stdout --output=nfssrv1 xvalidation.tpl             (will succeed)
#   % pan --stdout --output=nfsclt1 xvalidation.tpl             (will succeed)
#   % pan --stdout --output=nfsclt2 xvalidation.tpl             (will fail)
#
# $Id: xvalidation.tpl,v 1.5 2002/07/23 09:24:30 cons Exp $
#
##############################################################################

##############################################################################
#
# types definitions
#

template types;

# export record (roughly what is in /etc/exports)
define type export = {
  "path"    : string   description "path of the exported directory"
  "client"  : string   description "name of client allowed to mount it"
  "options" ? string[] description "list of exporting options like ro"
};

# mount record (roughly what is in /etc/fstab)
define type mount = {
  "device"  : string   description "device as understood by the mount command"
  "path"    : string   description "path of the mount point"
  "type"    : string   description "type of the mounted filesystem"
  "name"    ? string   description "name or label of this mount entry"
  "options" ? string[] description "list of mounting options like ro"
} with valid_mount(self);

# validation of a mount record (only nfs type records are checked)
define function valid_mount = {
  # the mount record is our only argument
  mount = argv[0];
  # we only care about NFS mounts, other types are considered OK
  if (mount["type"] != "nfs")
    return(true);
  # the device field will give us the NFS server and path
  result = matches(mount["device"], '^([\w\.\-]+):(.+)$');
  if (length(result) == 0)
    error("bad nfs device: " + mount["device"]);
  server = result[1];
  path   = result[2];
  # we now look at the server's exports list
  exports = value("//" + server + "/system/exports");
  i = 0;
  len = length(exports);
  while (i < len) {
    # we check if this export record is good for us by checking the client
    # field against object (i.e. the name of the current object template)
    # and the path; we want exact match and ignore the export/mount options
    if (exports[i]["client"] == object && exports[i]["path"] == path)
      return(true);
    i = i + 1;
  };
  # we haven't found any export record matching our needs, we complain:
  error("server " + server + " does not export " + path + " to " + object);
};

##############################################################################
#
# NFS server definition
#

object template nfssrv1;

# type settings
include types;
type "/system/exports" = export[];

# data for this host
"/system/exports" = list(
  nlist(                    # we export /home to hostx
    "path",    "/home",
    "client",  "hostx",
  ),
  nlist(                    # we export /home to nfsclt1, read-only
    "path",    "/home",
    "client",  "nfsclt1",
    "options", list("ro"),
  ),
);

##############################################################################
#
# NFS clients definitions
#

template client;

# type settings
include types;
type "/system/mounts" = mount[];

# data for this host
"/system/mounts" = list(
  nlist(                    # we mount /dev/hda1 as the root filesystem
    "device", "/dev/hda1",
    "path",   "/",
    "type",   "ext2",
  ),
  nlist(                    # we NFS mount /home from the server nfssrv1
    "device", "nfssrv1:/home",
    "path",   "/home",
    "type",   "nfs",
  ),
);

# first client: known by the server, compilation will succeed
object template nfsclt1;
include client;

# second client: unknown to the server, compilation will fail with:
# *** user error: server nfssrv1 does not export /home to nfsclt2
object template nfsclt2;
include client;
